@@ -20,8 +20,6 @@
 /**
  * SECTION:element-dfbvideosink
  *
- * <refsect2>
- * <para>
  * DfbVideoSink renders video frames using the
  * <ulink url="http://www.directfb.org/">DirectFB</ulink> library.
  * Rendering can happen in two different modes :
@@ -47,19 +45,19 @@
  * <listitem>
  *   <para>
  *   Embedded: this mode will render video frames in a 
- *   <link linkend="GstDfbVideoSink--surface">surface</link> provided by the
+ *   #GstDfbVideoSink:surface provided by the
  *   application developer. This is a more advanced usage of the element and
  *   it is required to integrate video playback in existing 
  *   <ulink url="http://www.directfb.org/">DirectFB</ulink> applications.
  *   </para>
  *   <para>
  *   When using this mode the element just renders to the
- *   <link linkend="GstDfbVideoSink--surface">surface</link> provided by the 
+ *   #GstDfbVideoSink:surface provided by the 
  *   application, that means it won't handle navigation events and won't resize
- *   the <link linkend="GstDfbVideoSink--surface">surface</link> to fit video
+ *   the #GstDfbVideoSink:surface to fit video
  *   frames geometry. Application has to implement the necessary code to grab
  *   informations about the negotiated geometry and resize there
- *   <link linkend="GstDfbVideoSink--surface">surface</link> accordingly.
+ *   #GstDfbVideoSink:surface accordingly.
  *   </para>
  * </listitem>
  * </itemizedlist>
@@ -67,20 +65,19 @@
  * optimize memory allocation time and handle reverse negotiation. Indeed if 
  * you insert an element like videoscale in the pipeline the video sink will
  * negotiate with it to try get a scaled video for either the fullscreen layout
- * or the application provided external
- * <link linkend="GstDfbVideoSink--surface">surface</link>.
- * </para>
+ * or the application provided external #GstDfbVideoSink:surface.
+ *
+ * <refsect2>
  * <title>Example application</title>
  * <para>
  * <include xmlns="http://www.w3.org/2003/XInclude" href="element-dfb-example.xml" />
  * </para>
+ * </refsect2>
+ * <refsect2>
  * <title>Example pipelines</title>
- * <para>
- * Here is a test pipeline to test the colorbalance interface :
- * <programlisting>
+ * |[
  * gst-launch -v videotestsrc ! dfbvideosink hue=20000 saturation=40000 brightness=25000
- * </programlisting>
- * </para>
+ * ]| test the colorbalance interface implementation in dfbvideosink
  * </refsect2>
  */
 
@@ -96,6 +93,7 @@
 #include "dfbvideosink.h"
 
 #include <string.h>
+#include <stdlib.h>
 
 /* Debugging category */
 GST_DEBUG_CATEGORY_STATIC (dfbvideosink_debug);
@@ -144,7 +142,9 @@ static void gst_dfbvideosink_surface_destroy (GstDfbVideoSink * dfbvideosink,
     GstDfbSurface * surface);
 
 static GstVideoSinkClass *parent_class = NULL;
+static GstBufferClass *surface_parent_class = NULL;
 
+#ifndef GST_DISABLE_GST_DEBUG
 static const char *
 gst_dfbvideosink_get_format_name (DFBSurfacePixelFormat format)
 {
@@ -193,6 +193,7 @@ gst_dfbvideosink_get_format_name (DFBSurfacePixelFormat format)
       return "UNKNOWN";
   }
 }
+#endif /* GST_DISABLE_GST_DEBUG */
 
 /* Creates miniobject and our internal surface */
 static GstDfbSurface *
@@ -328,18 +329,14 @@ gst_dfbvideosink_surface_destroy (GstDfbVideoSink * dfbvideosink,
     surface->surface = NULL;
   }
 
-  if (GST_BUFFER (surface)->malloc_data) {
-    g_free (GST_BUFFER (surface)->malloc_data);
-    GST_BUFFER (surface)->malloc_data = NULL;
-  }
-
   if (surface->dfbvideosink) {
     /* Release the ref to our sink */
     surface->dfbvideosink = NULL;
     gst_object_unref (dfbvideosink);
   }
 
-  return;
+  GST_MINI_OBJECT_CLASS (surface_parent_class)->finalize (GST_MINI_OBJECT
+      (surface));
 }
 
 static gpointer
@@ -584,10 +581,14 @@ gst_dfbvideosink_setup (GstDfbVideoSink * dfbvideosink)
         "DirectFB fullscreen");
     if (!dfbvideosink->dfb) {
       DFBGraphicsDeviceDescription hw_caps;
+      char *argv[] = { "-", "--dfb:quiet", NULL };
+      int argc = 2;
+      char **args;
 
       GST_DEBUG_OBJECT (dfbvideosink, "initializing DirectFB");
 
-      ret = DirectFBInit (0, NULL);
+      args = argv;
+      ret = DirectFBInit (&argc, &args);
 
       if (ret != DFB_OK) {
         GST_WARNING_OBJECT (dfbvideosink, "DirectFB initialization failed");
@@ -1540,7 +1541,7 @@ gst_dfbvideosink_show_frame (GstBaseSink * bsink, GstBuffer * buf)
 
   if (mem_cpy) {
     IDirectFBSurface *dest = NULL, *surface = NULL;
-    gpointer data;
+    guint8 *data;
     gint dest_pitch, src_pitch, line;
     GstStructure *structure;
 
@@ -1573,7 +1574,9 @@ gst_dfbvideosink_show_frame (GstBaseSink * bsink, GstBuffer * buf)
     /* Center / Clip */
     gst_video_sink_center_rect (src, dst, &result, FALSE);
 
-    res = surface->GetSubSurface (surface, (DFBRectangle *) & result, &dest);
+    res =
+        surface->GetSubSurface (surface, (DFBRectangle *) (void *) &result,
+        &dest);
     if (res != DFB_OK) {
       GST_WARNING_OBJECT (dfbvideosink, "failed when getting a sub surface");
       ret = GST_FLOW_UNEXPECTED;
@@ -1585,7 +1588,7 @@ gst_dfbvideosink_show_frame (GstBaseSink * bsink, GstBuffer * buf)
       dfbvideosink->layer->WaitForSync (dfbvideosink->layer);
     }
 
-    res = dest->Lock (dest, DSLF_WRITE, &data, &dest_pitch);
+    res = dest->Lock (dest, DSLF_WRITE, (void *) &data, &dest_pitch);
     if (res != DFB_OK) {
       GST_WARNING_OBJECT (dfbvideosink, "failed locking the external "
           "subsurface for writing");
@@ -1642,7 +1645,7 @@ gst_dfbvideosink_show_frame (GstBaseSink * bsink, GstBuffer * buf)
 
     if (dfbvideosink->hw_scaling) {
       dfbvideosink->primary->StretchBlit (dfbvideosink->primary,
-          surface->surface, NULL, (DFBRectangle *) & result);
+          surface->surface, NULL, (DFBRectangle *) (void *) &result);
     } else {
       DFBRectangle clip;
 
@@ -1891,6 +1894,8 @@ gst_dfbsurface_class_init (gpointer g_class, gpointer class_data)
 {
   GstMiniObjectClass *mini_object_class = GST_MINI_OBJECT_CLASS (g_class);
 
+  surface_parent_class = g_type_class_peek_parent (g_class);
+
   mini_object_class->finalize = (GstMiniObjectFinalizeFunction)
       gst_dfbsurface_finalize;
 }
@@ -1942,8 +1947,7 @@ gst_dfbvideosink_navigation_send_event (GstNavigation * navigation,
   GstDfbVideoSink *dfbvideosink = GST_DFBVIDEOSINK (navigation);
   GstEvent *event;
   GstVideoRectangle src, dst, result;
-  gint width, height;
-  double x, y;
+  double x, y, old_x, old_y;
   GstPad *pad = NULL;
 
   src.w = GST_VIDEO_SINK_WIDTH (dfbvideosink);
@@ -1957,8 +1961,8 @@ gst_dfbvideosink_navigation_send_event (GstNavigation * navigation,
   /* Our coordinates can be wrong here if we centered the video */
 
   /* Converting pointer coordinates to the non scaled geometry */
-  if (gst_structure_get_double (structure, "pointer_x", &x)) {
-    double old_x = x;
+  if (gst_structure_get_double (structure, "pointer_x", &old_x)) {
+    x = old_x;
 
     if (x >= result.x && x <= (result.x + result.w)) {
       x -= result.x;
@@ -1971,8 +1975,8 @@ gst_dfbvideosink_navigation_send_event (GstNavigation * navigation,
         "coordinate from %f to %f", old_x, x);
     gst_structure_set (structure, "pointer_x", G_TYPE_DOUBLE, x, NULL);
   }
-  if (gst_structure_get_double (structure, "pointer_y", &y)) {
-    double old_y = y;
+  if (gst_structure_get_double (structure, "pointer_y", &old_y)) {
+    y = old_y;
 
     if (y >= result.y && y <= (result.y + result.h)) {
       y -= result.y;
@@ -2225,6 +2229,8 @@ gst_dfbvideosink_finalize (GObject * object)
   if (dfbvideosink->setup) {
     gst_dfbvideosink_cleanup (dfbvideosink);
   }
+
+  G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 static void
@@ -2366,6 +2372,11 @@ gst_dfbvideosink_get_type (void)
       NULL,
       NULL,
     };
+    static const GInterfaceInfo colorbalance_info = {
+      (GInterfaceInitFunc) gst_dfbvideosink_colorbalance_init,
+      NULL,
+      NULL,
+    };
 
     dfbvideosink_type = g_type_register_static (GST_TYPE_VIDEO_SINK,
         "GstDfbVideoSink", &dfbvideosink_info, 0);
@@ -2374,6 +2385,8 @@ gst_dfbvideosink_get_type (void)
         GST_TYPE_IMPLEMENTS_INTERFACE, &iface_info);
     g_type_add_interface_static (dfbvideosink_type, GST_TYPE_NAVIGATION,
         &navigation_info);
+    g_type_add_interface_static (dfbvideosink_type, GST_TYPE_COLOR_BALANCE,
+        &colorbalance_info);
   }
 
   return dfbvideosink_type;
