@@ -556,11 +556,13 @@ metadataparse_exif_content_foreach_entry_func (ExifEntry * entry,
   MEUserData *meudata = (MEUserData *) user_data;
   GType type = G_TYPE_NONE;
   ExifByteOrder byte_order;
-  const gchar *tag = metadataparse_exif_get_tag_from_exif (entry->tag, &type);
+  const gchar *tag;
 
   /* We need the byte order */
   if (!entry || !entry->parent || !entry->parent->parent)
     return;
+
+  tag = metadataparse_exif_get_tag_from_exif (entry->tag, &type);
   byte_order = exif_data_get_byte_order (entry->parent->parent);
 
   if (metadataparse_handle_unit_tags (entry, meudata, byte_order))
@@ -694,16 +696,19 @@ metadataparse_exif_content_foreach_entry_func (ExifEntry * entry,
 
           /* DDD - degrees */
           value = (gdouble) rt->numerator / (gdouble) rt->denominator;
+          GST_DEBUG ("deg: %lu / %lu", (gulong) rt->numerator,
+              (gulong) rt->denominator);
           rt++;
 
           /* MM - minutes and SS - seconds */
-          if (rt->numerator % rt->denominator) {
-            value += (gdouble) rt->numerator / (gdouble) rt->denominator;
-          } else {
-            value += rt->numerator / rt->denominator;
-            rt++;
-            value += rt->numerator / rt->denominator;
-          }
+          GST_DEBUG ("min: %lu / %lu", (gulong) rt->numerator,
+              (gulong) rt->denominator);
+          value += (gdouble) rt->numerator / ((gdouble) rt->denominator * 60.0);
+          rt++;
+          GST_DEBUG ("sec: %lu / %lu", (gulong) rt->numerator,
+              (gulong) rt->denominator);
+          value +=
+              (gdouble) rt->numerator / ((gdouble) rt->denominator * 3600.0);
 
           /* apply sign */
           if (entry->tag == EXIF_TAG_GPS_LATITUDE) {
@@ -739,6 +744,7 @@ metadataparse_exif_content_foreach_entry_func (ExifEntry * entry,
 
 done:
   {
+#ifndef GST_DISABLE_GST_DEBUG
     char buf[2048];
     GST_LOG ("\n    Entry %p: %s (%s)\n"
         "      Size, Comps: %d, %d\n"
@@ -752,6 +758,7 @@ done:
         (int) (entry->components),
         exif_entry_get_value (entry, buf, sizeof (buf)),
         exif_tag_get_title (entry->tag), exif_tag_get_description (entry->tag));
+#endif
   }
   return;
 
@@ -1047,17 +1054,27 @@ metadatamux_exif_for_each_tag_in_list (const GstTagList * list,
           const ExifTag ref_tag = entry->tag == EXIF_TAG_GPS_LATITUDE ?
               EXIF_TAG_GPS_LATITUDE_REF : EXIF_TAG_GPS_LONGITUDE_REF;
 
+          /* DDD - degrees */
           rt->numerator = (gulong) v;
           rt->denominator = 1;
+          GST_DEBUG ("deg: %lf : %lu / %lu", v, (gulong) rt->numerator,
+              (gulong) rt->denominator);
           v -= rt->numerator;
           rt++;
 
-          rt->numerator = (gulong) (0.5 + v * 100.0);
-          rt->denominator = 100;
+          /* MM - minutes */
+          rt->numerator = (gulong) (v * 60.0);
+          rt->denominator = 1;
+          GST_DEBUG ("min: %lf : %lu / %lu", v, (gulong) rt->numerator,
+              (gulong) rt->denominator);
+          v -= ((gdouble) rt->numerator / 60.0);
           rt++;
 
-          rt->numerator = 0;
+          /* SS - seconds */
+          rt->numerator = (gulong) (0.5 + v * 3600.0);
           rt->denominator = 1;
+          GST_DEBUG ("sec: %lf : %lu / %lu", v, (gulong) rt->numerator,
+              (gulong) rt->denominator);
 
           if (entry->tag == EXIF_TAG_GPS_LONGITUDE) {
             GST_DEBUG ("longitude : %lf", value);
