@@ -26,14 +26,12 @@
 #include "gstcamerabin.h"
 
 G_BEGIN_DECLS
-    GstElement * gst_camerabin_preview_create_pipeline (GstCameraBin * camera,
-    GstCaps * caps);
+    gboolean gst_camerabin_preview_create_pipeline (GstCameraBin * camera);
 
-void gst_camerabin_preview_destroy_pipeline (GstCameraBin * camera,
-    GstElement * pipeline);
+void gst_camerabin_preview_destroy_pipeline (GstCameraBin * camera);
 
 GstBuffer *gst_camerabin_preview_convert (GstCameraBin * camera,
-    GstElement * pipeline, GstBuffer * buf);
+    GstBuffer * buf);
 
 G_END_DECLS
 #endif                          /* __CAMERABINPREVIEW_H__ */
