@@ -77,6 +77,33 @@ typedef enum
 GST_DEBUG_CATEGORY_STATIC (gstflupsdemux_debug);
 #define GST_CAT_DEFAULT (gstflupsdemux_debug)
 
+/* elementfactory information */
+static GstElementDetails flups_demux_details = {
+  "The Fluendo MPEG Program Stream Demuxer",
+  "Codec/Demuxer",
+  "Demultiplexes MPEG Program Streams",
+  "Wim Taymans <wim@fluendo.com>"
+};
+
+#define GST_TYPE_FLUPS_DEMUX_SYNC          (gst_flups_demux_sync_get_type ())
+GType
+gst_flups_demux_sync_get_type (void)
+{
+  static GType sync_type = 0;
+  static GEnumValue sync_types[] = {
+    {GST_FLUPS_DEMUX_SYNC_AUTO, "Auto-select method to synchronize streams",
+        "auto"},
+    {GST_FLUPS_DEMUX_SYNC_SCR, "Use SCR to synchronize streams", "scr"},
+    {GST_FLUPS_DEMUX_SYNC_DTS, "Use DTS to synchronize streams", "dts"},
+    {0, NULL, NULL}
+  };
+
+  if (sync_type == 0)
+    sync_type = g_enum_register_static ("GstPSDemuxSync", sync_types);
+
+  return sync_type;
+}
+
 /* MPEG2Demux signals and args */
 enum
 {
@@ -214,9 +241,7 @@ gst_flups_demux_base_init (GstFluPSDemuxClass * klass)
   gst_element_class_add_pad_template (element_class, klass->private_template);
   gst_element_class_add_pad_template (element_class, klass->sink_template);
 
-  gst_element_class_set_details_simple (element_class,
-      "The Fluendo MPEG Program Stream Demuxer", "Codec/Demuxer",
-      "Demultiplexes MPEG Program Streams", "Wim Taymans <wim@fluendo.com>");
+  gst_element_class_set_details (element_class, &flups_demux_details);
 }
 
 static void
@@ -738,52 +763,43 @@ gst_flups_demux_clear_times (GstFluPSDemux * demux)
 
 static inline void
 gst_flups_demux_send_segment_updates (GstFluPSDemux * demux,
-    GstClockTime new_start)
+    GstClockTime new_time)
 {
-  GstClockTime base_time, stop, time;
+  /* Advance all lagging streams by sending a segment update */
   gint i, count = demux->found_count;
   GstEvent *event = NULL;
 
-  /* Advance all lagging streams by sending a segment update */
-  if ((base_time = demux->base_time) == GST_CLOCK_TIME_NONE)
-    base_time = 0;
+  /* FIXME: Handle reverse playback */
 
-  stop = demux->src_segment.stop;
-  if (stop != GST_CLOCK_TIME_NONE)
-    stop += base_time;
-
-  if (new_start > stop)
+  if (new_time > demux->src_segment.stop)
     return;
 
-  time = demux->src_segment.time;
-  time += new_start - (demux->src_segment.start + base_time);
-
-  /* FIXME: Handle reverse playback */
   for (i = 0; i < count; i++) {
     GstFluPSStream *stream = demux->streams_found[i];
 
     if (stream) {
       if (stream->last_ts == GST_CLOCK_TIME_NONE ||
-          stream->last_ts < demux->src_segment.start + base_time)
-        stream->last_ts = demux->src_segment.start + base_time;
-
-      if (stream->last_ts + stream->segment_thresh < new_start) {
+          stream->last_ts < demux->src_segment.start)
+        stream->last_ts = demux->src_segment.start;
+      if (stream->last_ts + stream->segment_thresh < new_time) {
 #if 0
         g_print ("Segment update to pad %s time %" GST_TIME_FORMAT " stop now %"
             GST_TIME_FORMAT "\n", GST_PAD_NAME (stream->pad),
-            GST_TIME_ARGS (new_start), GST_TIME_ARGS (demux->src_segment.stop));
+            GST_TIME_ARGS (new_time), GST_TIME_ARGS (demux->src_segment.stop));
 #endif
         GST_DEBUG_OBJECT (demux,
             "Segment update to pad %s time %" GST_TIME_FORMAT,
-            GST_PAD_NAME (stream->pad), GST_TIME_ARGS (new_start));
+            GST_PAD_NAME (stream->pad), GST_TIME_ARGS (new_time));
         if (event == NULL) {
           event = gst_event_new_new_segment_full (TRUE,
               demux->src_segment.rate, demux->src_segment.applied_rate,
-              GST_FORMAT_TIME, new_start, stop, time);
+              GST_FORMAT_TIME, new_time,
+              demux->src_segment.stop,
+              demux->src_segment.time + (new_time - demux->src_segment.start));
         }
         gst_event_ref (event);
         gst_pad_push_event (stream->pad, event);
-        stream->last_seg_start = stream->last_ts = new_start;
+        stream->last_seg_start = stream->last_ts = new_time;
         stream->need_segment = FALSE;
       }
     }
@@ -1021,12 +1037,6 @@ gst_flups_demux_do_seek (GstFluPSDemux * demux, GstSegment * seeksegment)
   guint64 scr_rate_n = demux->last_scr_offset - demux->first_scr_offset;
   guint64 scr_rate_d = demux->last_scr - demux->first_scr;
 
-  /* In some clips the PTS values are completely unaligned with SCR values.
-   * To improve the seek in that situation we apply a factor considering the
-   * relationship between last PTS and last SCR */
-  if (demux->last_scr > demux->last_pts)
-    scr = gst_util_uint64_scale (scr, demux->last_scr, demux->last_pts);
-
   scr = MIN (demux->last_scr, scr);
   scr = MAX (demux->first_scr, scr);
   fscr = scr;
@@ -1472,8 +1482,8 @@ gst_flups_demux_parse_pack_start (GstFluPSDemux * demux)
   /* skip start code */
   data += 4;
 
-  scr1 = GST_READ_UINT32_BE (data);
-  scr2 = GST_READ_UINT32_BE (data + 4);
+  scr1 = GUINT32_FROM_BE (*(guint32 *) data);
+  scr2 = GUINT32_FROM_BE (*(guint32 *) (data + 4));
 
   /* fixed length to begin with, start code and two scr values */
   length = 8 + 4;
@@ -1509,8 +1519,7 @@ gst_flups_demux_parse_pack_start (GstFluPSDemux * demux)
     /* We keep the offset of this scr */
     demux->cur_scr_offset = demux->adapter_offset + 12;
 
-    GST_DEBUG_OBJECT (demux, "SCR: 0x%08" G_GINT64_MODIFIER "x SCRE: 0x%08x",
-        scr, scr_ext);
+    GST_DEBUG_OBJECT (demux, "SCR: 0x%08llx SCRE: 0x%08x", scr, scr_ext);
 
     if (scr_ext) {
       scr = (scr * 300 + scr_ext % 300) / 300;
@@ -1519,7 +1528,7 @@ gst_flups_demux_parse_pack_start (GstFluPSDemux * demux)
        to DTS/PTS, that also implies 1 tick rounding error */
     data += 6;
     /* PMR:22 ! :2==11 ! reserved:5 ! stuffing_len:3 */
-    next32 = GST_READ_UINT32_BE (data);
+    next32 = (GUINT32_FROM_BE ((*(guint32 *) data)));
     if (G_UNLIKELY ((next32 & 0x00000300) != 0x00000300))
       goto lost_sync;
 
@@ -2202,8 +2211,8 @@ gst_flups_demux_scan_ts (GstFluPSDemux * demux, const guint8 * data,
   /* skip start code */
   data += 4;
 
-  scr1 = GST_READ_UINT32_BE (data);
-  scr2 = GST_READ_UINT32_BE (data + 4);
+  scr1 = GUINT32_FROM_BE (*(guint32 *) data);
+  scr2 = GUINT32_FROM_BE (*(guint32 *) (data + 4));
 
   /* start parsing the stream */
   if ((*data & 0xc0) == 0x40) {
@@ -2235,7 +2244,7 @@ gst_flups_demux_scan_ts (GstFluPSDemux * demux, const guint8 * data,
        to DTS/PTS, that also implies 1 tick rounding error */
     data += 6;
     /* PMR:22 ! :2==11 ! reserved:5 ! stuffing_len:3 */
-    next32 = GST_READ_UINT32_BE (data);
+    next32 = (GUINT32_FROM_BE ((*(guint32 *) data)));
     if ((next32 & 0x00000300) != 0x00000300)
       goto beach;
 
